/* ulam2 - spiral to given {N} */

// +w960 +h600 +a0.1 +am3 +ac0.99 +ag1 -j +q11
// +w480 +h300 +a0.1 +am3 +ag1 -j +q11 +kc +rtr +kla

#version 3.8;

global_settings {assumed_gamma 1}

/* 'to1M' data not included, due to size.  can send either xz compressed
 * version or generator (Tcl script) on request.
 */
#if (!defined(N))
  #declare N = 1;
  #declare csv_file_ = "to4K.csv";
#elseif (1 > N | 1048576 < N)
  #error "oops, out-of-range value 'N', must be 1,..,1048576."
#elseif (262144 < N)
  #declare csv_file_ = "to1M.csv";
#elseif (4096 < N)
  #declare csv_file_ = "to256K.csv";
#else
  #declare csv_file_ = "to4K.csv";
#end

#declare heights_    = true;
#declare use_rtr_    = true;
#declare verbose_    = true;
#declare with_bot_   = false;
#declare with_plane_ = false;

#include "filed.inc"
#include "foreach.inc"
#include "pvars.inc"
#include "ruled.inc"

light_source {<-1,1,-1> * 1e3 color srgb 1 parallel}
light_source {<1,1,-1> * 1e3 color srgb 1 parallel}

#declare textures_ = array [2] {
  texture {pigment {color rgb <0,0,1>} finish {ambient 0 emission 1}},
  texture {pigment {color rgb <1,1,0>} finish {ambient 0 emission 1}}
};

/* file/record layout.  all bot moves pre-calculated, and complete state
 * included although redundant.  (isPrime) where 2 == {nfactors}.
 * csv: {pv1,pv2}
 *   pv1 <x,y,nfactors,facing>
 *   pv2 <turns,stride,leg,n>
 */
#declare items_ = dictionary {
  .File     : csv_file_,
  .Access   : "r",
  .Fields   : array [2] {"V40","V40"},
  .Range    : array [2] {1,N},
  .hasNames : true,
  .Verbose  : verbose_
};
Filed(items_)

/* show #factors as separate heights, or "flat" line */
#if (heights_)

  #macro m_mkItem(i_,elem_)
    #local (x_,y_,nf_,junk_) = pvUnpack(elem_);
    #local p_ = <(x_ + .5),0,(y_ + .5)>;
    #local t_ = <(x_ + .5),nf_,(y_ + .5)>;
    #if (2 = nf_)
      cylinder {p_, t_, .15 texture {textures_[1]}}
    #else
      cone {p_, .15, t_, 0 texture {textures_[0]}}
    #end
  #end

  Foreach(items_.Data, dictionary {.Macro: "m_mkItem"})

#else

  #macro m_mkItems(i_,elem_,arg_)
    #local (x_,y_,nf_,junk_) = pvUnpack(elem_);
    #local p_ = <(x_ + .5),.15,(y_ + .5)>;
    union {
      cylinder {arg_.last_, p_, .075}
      sphere {p_, .075}
      texture {textures_[0]}
    }
    #if (2 = nf_)
      sphere {p_, .125 texture {textures_[1]}}
    #end
    #local arg_.last_ = p_;
  #end

  #declare tmp_ = dictionary {.last_: <.5,.05,.5>};

  Foreach(items_.Data, dictionary {.Macro: "m_mkItems", .Arg: "tmp_"})

#end

#if (with_bot_)
#declare (x_,y_,junk_,face_) = pvUnpack(items_.Data[N-1][0]);
  object {
    merge {
      difference {
        cylinder {<0,0,0>,<0,1.25,0>,.30}
        text {
          ttf "cyrvetic.ttf","0",1.5,0
          scale <.9,.9,1.5>
          translate <-.22375,.2,-.5>
        }
      }
      cone {<0,.95,0>,.25,<0,.85,.65>,0 scale <.5,1,1>}
    }
    pigment {color rgb <1,0,0>}
    rotate <0,90*face_,0>
    translate <(x_+.5),0,(y_+.5)>
  }
#end

#if (with_plane_)
  Ruled(dictionary {
    .norm : <0,1,0>,
    .offs : -.05,
    .divs : 1,
    .bg   : pigment {color rgbt .975},
    .dim  : false
  })
#end

#declare z_ = -(5 + int(sqrt(N)));

#declare camera_ = camera {
  location <0,abs(z_ / 2),z_>
  right x * (16/10)
  up y
  angle 65
  look_at <0,0,0>
};

#if (use_rtr_)
  #for (i_,0,359)
    camera {camera_ rotate y*i_}
  #end
#else
  camera {camera_}
#end

